#
# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#  Contributor(s): Sergey Sharybin,
#                  Francesco Siddi
#
# ##### END GPL LICENSE BLOCK #####
#

#
# This script updates paths to libraries, images, sounds and movie clips used in
# .blend files after this files were moved using `svn mv` command
#

import sys
import os
import subprocess
import re
import urllib
import blendfile as blendfile

PROJECT = "mango"


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'

    def disable(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''


def abspath(path, start):
    if path.startswith("//"):
        return os.path.join(start, path[2:])

    return path


def relpath(path, start):
    if not path.startswith("//"):
        return "//" + os.path.relpath(path, start)

    return path


def renameFilepath(filepath, start, renameMap):
    abs_path = os.path.realpath(abspath(filepath, start))

    if abs_path in renameMap:
        new_filepath = relpath(renameMap[abs_path], start)

        return new_filepath

    return filepath


def renameFilepathDatablock(block, prop, start, renameMap):
    val = block.Get(prop).split("\0")[0]
    new_val = renameFilepath(val, start, renameMap)
    if val != new_val:
        print(bcolors.WARNING +
            "    Changing %s from %s to %s" % (prop, val, new_val) +
            bcolors.ENDC)
        val = block.Set(prop, new_val)


def processBlendFile(filepath, renameMap):
    rules = {"LI": {"name": "library",
                    "caption": "name",
                    "check": ["name"]},
             "IM": {"name": "image",
                    "check": ["name"]},
             "SO": {"name": "sound",
                    "check": ["name"]},
             "MC": {"name": "movieclip",
                    "check": ["name"]}}

    print(bcolors.HEADER + "Processing BLEND %s" % (filepath) + bcolors.ENDC)

    start = os.path.dirname(filepath)
    blend = blendfile.openBlendFile(filepath, access="r+b")

    for blk_id in rules:
        rule = rules[blk_id]
        for block in blend.FindBlendFileBlocksWithCode(blk_id):
            cap = rule["caption"] if "caption" in rule else "id.name"
            name = block.Get(cap).split("\0")[0]
            print("  Processing %s %s" % (rule["name"], name))

            for prop in rule["check"]:
                renameFilepathDatablock(block, prop, start, renameMap)

    blend.close()


def findRenamedFiles(repo):
    renameMap = {}

    p = subprocess.Popen(["svn", "st", repo], stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()

    for line in stdout.split("\n"):
        line = re.sub("\s+", " ", line)

        if line.startswith("A +"):
            f = line[4:]
            p = subprocess.Popen(["svn", "info", f], stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            stdout, stderr = p.communicate()

            root = None
            copied_from = None
            url = None
            for info_line in stdout.split("\n"):
                if info_line.startswith("Repository Root"):
                    root = info_line[17:]
                elif info_line.startswith("Copied From URL"):
                    copied_from = info_line[17:]
                elif info_line.startswith("URL"):
                    url = info_line[5:]

            if root and copied_from and url:
                old_file = urllib.url2pathname(copied_from.replace(root, ""))
                new_file = urllib.url2pathname(url.replace(root, ""))

                old_file = os.path.join(repo, old_file[1:])
                new_file = os.path.join(repo, new_file[1:])

                renameMap[old_file] = new_file

    return renameMap


if __name__ == "__main__":
    repo = "/%s" % (PROJECT)

    print("Generating list of renamed files")
    renameMap = findRenamedFiles(repo)

    print("Updating paths for all .blend files in a production")
    for root, dirs, files in os.walk(repo):
        if root.find('.svn') == -1:
            for f in files:
                if f.endswith(".blend"):
                    processBlendFile(os.path.join(root, f), renameMap)
